@@ -39,6 +39,8 @@ sub new{
         'data' => {},
         'binMimeType' => undef ,
         'bin' => undef ,
+	'stream' => undef,
+	'multipart_stream' => undef,
     };
     return bless $self , $class ;
 }
@@ -60,6 +62,7 @@ sub cleanup{
             delete $self->{$key} ;
         }
     }
+    
 }
 
 =head2 status
@@ -92,4 +95,20 @@ Get/Set the MIME type of the binary content.
 
 =cut
 
+=head2 stream
+
+Get/Set the L<Apache2::REST::Stream> to render this response as a stream
+of data.
+
+Setting this will trigger a '_Stream' version of writers to be used.
+
+=head2 multipart_stream
+
+Get/Set the L<Apache2::REST::Stream> to render this response as a stream
+of data composed of multipart response parts.
+
+Setting this will trigger a '_multipart' version of writers to be used.
+
+=cut
+
 1;
